@@ -22,12 +22,12 @@ def wait_channel(channel):
 
 if __name__ == '__main__':
     def cb(result, error):
-        print result
-        print error
+        print(result)
+        print(error)
     channel = pycares.Channel()
     channel.gethostbyname('google.com', socket.AF_INET, cb)
     channel.query('google.com', pycares.QUERY_TYPE_A, cb)
     channel.query('sip2sip.info', pycares.QUERY_TYPE_SOA, cb)
     wait_channel(channel)
-    print "Done!"
+    print('Done!')
 
